/*
 * Copyright 2026 Element Creations Ltd.
 *
 * SPDX-License-Identifier: AGPL-3.0-only OR GPL-3.0-only OR LicenseRef-Element-Commercial
 * Please see LICENSE files in the repository root for full details.
 */

.content svg[data-state="supported"] {
    color: inherit;
}

.content svg[data-state="unsupported"] {
    color: var(--cpd-color-icon-critical-primary);
}
