/*
 * Copyright 2025 New Vector Ltd.
 *
 * SPDX-License-Identifier: AGPL-3.0-only OR GPL-3.0-only OR LicenseRef-Element-Commercial
 * Please see LICENSE files in the repository root for full details.
 */

.pillInput {
    background-color: var(--cpd-color-bg-subtle-secondary);
    border-radius: 20px;
    padding: var(--cpd-space-2x) var(--cpd-space-3x) var(--cpd-space-2x) var(--cpd-space-3x);
    /* To match pill height in order to avoid the PillInput to grow when a pill is inserted */
    min-height: 28px;
}

.pillInput:has(.input:focus) {
    outline: var(--cpd-border-width-1) solid var(--cpd-color-gray-1400);
}

.input {
    all: unset;
    width: 100%;
    flex: 1;
    color: var(--cpd-color-text-primary);
}

.input::placeholder {
    color: var(--cpd-color-text-secondary);
    font: var(--cpd-font-body-md-regular);
}

.largerInput {
    padding: var(--cpd-space-2x) 0;
}
