/*
Copyright 2024 New Vector Ltd.
Copyright 2021 The Matrix.org Foundation C.I.C.

SPDX-License-Identifier: AGPL-3.0-only OR GPL-3.0-only OR LicenseRef-Element-Commercial
Please see LICENSE files in the repository root for full details.
*/

/*
    This file is raw-imported (imported as plain text) for the export bundle, which is the reason for the .css format and the colours being hard-coded hard-coded.
*/

html,
body {
    font-size: var(--cpd-font-size-root) !important;
}

#snackbar {
    display: flex;
    visibility: hidden;
    min-width: 250px;
    margin-left: -125px;
    background-color: #333;
    color: #fff;
    text-align: center;
    position: fixed;
    z-index: 1;
    left: 50%;
    bottom: 30px;
    font-size: 17px;
    padding: 6px 16px;
    font-family:
        -apple-system,
        BlinkMacSystemFont,
        avenir next,
        avenir,
        segoe ui,
        helvetica neue,
        helvetica,
        Ubuntu,
        roboto,
        noto,
        arial,
        sans-serif;
    font-weight: 400;
    line-height: 1.43;
    border-radius: 4px;
    letter-spacing: 0.01071em;
}

#snackbar.mx_show {
    visibility: visible;
    -webkit-animation:
        mx_snackbar_fadein 0.5s,
        mx_snackbar_fadeout 0.5s 2.5s;
    animation:
        mx_snackbar_fadein 0.5s,
        mx_snackbar_fadeout 0.5s 2.5s;
}

a.mx_reply_anchor {
    cursor: pointer;
}

@-webkit-keyframes mx_snackbar_fadein {
    from {
        bottom: 0;
        opacity: 0;
    }
    to {
        bottom: 30px;
        opacity: 1;
    }
}

@keyframes mx_snackbar_fadein {
    from {
        bottom: 0;
        opacity: 0;
    }
    to {
        bottom: 30px;
        opacity: 1;
    }
}

@-webkit-keyframes mx_snackbar_fadeout {
    from {
        bottom: 30px;
        opacity: 1;
    }
    to {
        bottom: 0;
        opacity: 0;
    }
}

@keyframes mx_snackbar_fadeout {
    from {
        bottom: 30px;
        opacity: 1;
    }
    to {
        bottom: 0;
        opacity: 0;
    }
}

* {
    scroll-behavior: smooth !important;
}

.mx_Export_EventWrapper:target {
    background: white;
    animation: mx_event_highlight_animation 2s linear;
}

@keyframes mx_event_highlight_animation {
    0%,
    100% {
        background: white;
    }
    50% {
        background: #e3e2df;
    }
}

.mx_RoomHeader {
    display: flex;
    flex-direction: row;
    align-items: center;
    justify-content: start;
    gap: var(--cpd-space-3x);
}

.mx_ReplyChain_Export {
    margin-top: 0;
    margin-bottom: 5px;
}

.mx_RedactedBody,
.mx_HiddenBody {
    padding-left: unset;
}

img {
    white-space: nowrap;
    overflow: hidden;
}

.mx_MatrixChat {
    max-width: 100%;
}
