/*
 * Copyright 2026 Element Creations Ltd.
 *
 * SPDX-License-Identifier: AGPL-3.0-only OR GPL-3.0-only OR LicenseRef-Element-Commercial
 * Please see LICENSE files in the repository root for full details.
 */

.content {
    padding: 0 25px;
    cursor: pointer;

    h2 {
        flex: 0 0 auto;
        margin: 0;
        font-size: inherit;
        text-transform: capitalize;
    }

    svg {
        align-self: center;
        width: 16px;
        height: 16px;
        color: var(--cpd-color-icon-secondary);
    }
}
