/*
 * Copyright 2026 Element Creations Ltd.
 *
 * SPDX-License-Identifier: AGPL-3.0-only OR GPL-3.0-only OR LicenseRef-Element-Commercial
 * Please see LICENSE files in the repository root for full details.
 */

.toast {
    position: absolute;
    bottom: var(--cpd-space-4x);
    left: var(--cpd-space-3x);
    right: var(--cpd-space-3x);
    width: fit-content;
    margin: 0 auto;
}
