/*
 * Copyright 2026 Element Creations Ltd.
 *
 * SPDX-License-Identifier: AGPL-3.0-only OR GPL-3.0-only OR LicenseRef-Element-Commercial
 * Please see LICENSE files in the repository root for full details.
 */

.toolbar {
    gap: var(--cpd-space-0-5x);
    border-radius: 8px;
    background-color: var(--cpd-color-bg-canvas-default);
    border: var(--cpd-border-width-1) solid var(--cpd-color-border-disabled);

    .toolbar_item {
        align-items: center;
        justify-content: center;
        padding: 0;
        min-block-size: 0;

        margin: 3px;
        border-radius: 6px;

        &:hover {
            background-color: var(--cpd-color-bg-subtle-secondary);
            z-index: 1;
        }
    }

    .toolbar_item[data-presentation="icon"] {
        block-size: 28px;
        inline-size: 28px;

        color: var(--cpd-color-icon-secondary);
    }

    .toolbar_item[data-presentation="label"] {
        padding-inline-start: var(--cpd-space-2x);
        padding-inline-end: var(--cpd-space-2x);

        font: var(--cpd-font-body-md-regular);
        text-decoration: none;
        white-space: nowrap;
    }
}
