/*
 * Copyright 2026 Element Creations Ltd.
 *
 * SPDX-License-Identifier: AGPL-3.0-only OR GPL-3.0-only OR LicenseRef-Element-Commercial
 * Please see LICENSE files in the repository root for full details.
 */

.content {
    color: var(--cpd-color-text-secondary); /* override anchor color */
    font-size: var(--cpd-font-size-body-xs);
    font-variant-numeric: tabular-nums;
    display: inline-block;
    white-space: nowrap;
    user-select: none;
    text-decoration: none;
}
