/*
 * Copyright 2025 New Vector Ltd.
 *
 * SPDX-License-Identifier: AGPL-3.0-only OR GPL-3.0-only OR LicenseRef-Element-Commercial
 * Please see LICENSE files in the repository root for full details.
 */

.mediaBody {
    background-color: var(--cpd-color-bg-subtle-secondary);
    border-radius: 12px;
    max-width: 243px; /* use max-width instead of width so it fits within right panels */

    font: var(--cpd-font-body-md-regular);
    color: var(--cpd-color-text-secondary);

    padding: var(--cpd-space-1-5x) var(--cpd-space-3x);
}
