/*
 * Copyright 2026 Element Creations Ltd.
 *
 * SPDX-License-Identifier: AGPL-3.0-only OR GPL-3.0-only OR LicenseRef-Element-Commercial
 * Please see LICENSE files in the repository root for full details.
 */

.timelineSeparator {
    clear: both;
    margin: var(--cpd-space-1x) 0;
    font: var(--cpd-font-body-md-regular);
    letter-spacing: var(--cpd-font-letter-spacing-body-md);
    color: var(--cpd-color-text-primary);
}

.timelineSeparator > hr {
    flex: 1 1 0;
    height: 0;
    border: none;
    border-bottom: 1px solid var(--cpd-color-gray-400);
}
