/*
 * Copyright 2025 New Vector Ltd.
 *
 * SPDX-License-Identifier: AGPL-3.0-only OR GPL-3.0-only OR LicenseRef-Element-Commercial
 * Please see LICENSE files in the repository root for full details.
 */

.pill {
    background-color: var(--cpd-color-bg-action-primary-rest);
    padding: var(--cpd-space-1x) var(--cpd-space-1-5x) var(--cpd-space-1x) var(--cpd-space-1x);
    border-radius: 99px;
}

.label {
    color: var(--cpd-color-text-on-solid-primary);
    font: var(--cpd-font-body-sm-medium);
}
