/*
 * Copyright 2026 Element Creations Ltd.
 *
 * SPDX-License-Identifier: AGPL-3.0-only OR GPL-3.0-only OR LicenseRef-Element-Commercial
 * Please see LICENSE files in the repository root for full details.
 */

.pinnedMessageBadge {
    position: relative;
    display: flex;
    align-items: center;
    gap: var(--cpd-space-1x);
    padding: var(--cpd-space-1x) var(--cpd-space-3x) var(--cpd-space-1x) var(--cpd-space-1x);
    font: var(--cpd-font-body-xs-medium);
    background-color: var(--cpd-color-bg-subtle-secondary);
    color: var(--cpd-color-text-secondary);
    border: 1px solid var(--cpd-color-alpha-gray-400);
    border-radius: 99px;

    svg {
        fill: var(--cpd-color-icon-secondary);
    }
}
