/*
 * Copyright 2025 Element Creations Ltd.
 *
 * SPDX-License-Identifier: AGPL-3.0-only OR GPL-3.0-only OR LicenseRef-Element-Commercial
 * Please see LICENSE files in the repository root for full details.
 */

.skeleton {
    position: relative;
    margin-left: 4px;
    height: 100%;
    flex: 1;
}

.skeleton::before {
    background-color: var(--cpd-color-bg-subtle-secondary);
    width: 100%;
    height: 100%;
    content: "";
    position: absolute;
    mask-repeat: repeat-y;
    mask-size: auto 96px;
    mask-image: url("./assets/skeleton.svg");
}
