/*
Copyright 2025 Element Creations Ltd.

SPDX-License-Identifier: AGPL-3.0-only OR GPL-3.0-only OR LicenseRef-Element-Commercial
Please see LICENSE files in the repository root for full details.
*/

import { type TranslationKey as _TranslationKey } from "matrix-web-i18n";

import type Translations from "../i18n/strings/en_EN.json";

declare global {
    type TranslationKey = _TranslationKey<typeof Translations>;
}
