/*
Copyright 2025 New Vector Ltd.

SPDX-License-Identifier: AGPL-3.0-only OR LicenseRef-Element-Commercial
Please see LICENSE files in the repository root for full details.
*/

declare global {
    // eslint-disable-next-line no-var
    var __VERSION__: string; // injected by vite
}

export {};
