/*
 * Copyright 2026 Element Creations Ltd.
 *
 * SPDX-License-Identifier: AGPL-3.0-only OR GPL-3.0-only OR LicenseRef-Element-Commercial
 * Please see LICENSE files in the repository root for full details.
 */

.button {
    svg {
        transition: transform 0.1s linear;
    }
}

.button[aria-expanded="true"] {
    svg {
        transform: rotate(180deg);
    }
}
