/*
 * Copyright 2025 New Vector Ltd.
 *
 * SPDX-License-Identifier: AGPL-3.0-only OR GPL-3.0-only OR LicenseRef-Element-Commercial
 * Please see LICENSE files in the repository root for full details.
 */

.avatarWithDetails {
    display: flex;
    align-items: center;

    border-radius: 12px;
    background-color: var(--cpd-color-gray-200);
    padding: var(--cpd-space-3x);
    gap: var(--cpd-space-2x);

    .title {
        display: inline-block;

        font-weight: var(--cpd-font-weight-semibold);
        font-size: var(--cpd-font-size-body-md);

        overflow: hidden;
        text-overflow: ellipsis;
        white-space: nowrap;
    }

    .details {
        font-size: var(--cpd-font-size-body-sm);
        color: var(--cpd-color-text-secondary);
    }
}
