/*
 * Copyright 2025 Element Creations Ltd.
 *
 * SPDX-License-Identifier: AGPL-3.0-only OR GPL-3.0-only OR LicenseRef-Element-Commercial
 * Please see LICENSE files in the repository root for full details.
 */

.roomListPrimaryFilters {
    padding: 0 var(--cpd-space-4x) var(--cpd-space-5x) var(--cpd-space-3x);
}

/* Hide filters that are wrapping when collapsed */
.roomListPrimaryFilters :global(.wrapping) {
    display: none;
}

.list {
    /**
     * The InteractionObserver needs the height to be set to work properly.
     */
    height: 100%;
    flex: 1;
    user-select: none; /* currently supported by Chrome, Edge, Opera and Firefox */
}

/* IconButton styles for chevron */
.iconButton svg {
    transition: transform 0.1s linear;
}

.iconButton[aria-expanded="true"] svg {
    transform: rotate(180deg);
}
