.root {
    overflow: hidden;
}

.container {
    overflow: hidden;
    border-radius: var(--MBody-border-radius);
}

.video {
    display: block;
    width: 100%;
    height: 100%;
}

.hiddenButton {
    border: none;
    width: 100%;
    height: 100%;
    padding: 0;
    inset: 0;
    display: flex;
    align-items: center;
    justify-content: center;
    text-align: center;
    cursor: pointer;
}

.hiddenButtonContent {
    display: flex;
    align-items: center;
}

.hiddenButtonContent > svg {
    margin-top: auto;
    margin-bottom: auto;
}

.loadingContainer {
    display: flex;
    align-items: center;
    justify-content: center;
    width: 100%;
    height: 100%;
}

.error {
    display: inline-flex;
    align-items: center;
    gap: var(--cpd-space-1x);
}
