/*
Copyright 2024 New Vector Ltd.
Copyright 2023 The Matrix.org Foundation C.I.C.

SPDX-License-Identifier: AGPL-3.0-only OR GPL-3.0-only OR LicenseRef-Element-Commercial
Please see LICENSE files in the repository root for full details.
*/

.box-flex {
    flex: var(--mx-box-flex, unset);
}

.box-shrink {
    flex-shrink: var(--mx-box-shrink, unset);
}

.box-grow {
    flex-grow: var(--mx-box-grow, unset);
}
