/*
 * Copyright (c) 2025 Element Creations Ltd.
 *
 * SPDX-License-Identifier: AGPL-3.0-only OR GPL-3.0-only OR LicenseRef-Element-Commercial
 * Please see LICENSE files in the repository root for full details.
 */

.container {
    color: var(--cpd-color-text-primary);
}

.container svg {
    color: var(--cpd-color-text-primary);
}

.secondaryAction.secondaryAction[data-kind="secondary"] > svg {
    color: var(--cpd-color-text-secondary);
}

.primaryAction.primaryAction[data-kind="primary"] > svg {
    color: var(--cpd-color-text-on-solid-primary);
}

.title {
    color: var(--cpd-color-text-critical-primary);
}

.description {
    color: var(--cpd-color-text-primary);
}
