/*
 * Copyright 2025 Element Creations Ltd.
 *
 * SPDX-License-Identifier: AGPL-3.0-only OR GPL-3.0-only OR LicenseRef-Element-Commercial
 * Please see LICENSE files in the repository root for full details.
 */

.genericPlaceholder {
    align-self: center;
    /** It should take 2/3 of the width **/
    width: 66%;
    /** It should be positioned at 1/3 of the height **/
    padding-top: 33%;
}

.title {
    font: var(--cpd-font-body-lg-semibold);
    text-align: center;
}

.description {
    font: var(--cpd-font-body-sm-regular);
    color: var(--cpd-color-text-secondary);
    text-align: center;
}

.defaultPlaceholder {
    margin-top: var(--cpd-space-4x);
}

.genericPlaceholder button {
    width: 100%;
}
