/*
 * Copyright 2026 Element Creations Ltd.
 *
 * SPDX-License-Identifier: AGPL-3.0-only OR GPL-3.0-only OR LicenseRef-Element-Commercial
 * Please see LICENSE files in the repository root for full details.
 */

.root {
    overflow-y: hidden;
    overflow-x: hidden;
}

.text,
.caption {
    white-space: pre-wrap;
}

.notice {
    white-space: pre-wrap;
    color: var(--cpd-color-text-secondary);
}

.emote {
    white-space: pre-wrap;
    text-align: start;
}

.annotated {
    display: flex;
}

.annotatedInline {
    display: inline-flex;
}

.annotation {
    user-select: none;
    display: inline-block;
    margin-inline-start: 9px; /* Preserve legacy EventTile spacing for inline annotations like (edited) */
    font-size: 12px; /* Match the legacy EventTile edited-marker size */
    color: var(--cpd-color-text-secondary);
}

.editedMarker {
    appearance: none;
    background: none;
    border: none;
    height: max-content;
    padding: 0;
    cursor: pointer;
    font-family: inherit;
    font-weight: inherit;
    line-height: inherit;
}

.bodyLink,
.bodyAction {
    color: inherit;
    text-decoration: inherit;
}

.bodyAction {
    appearance: none;
    background: none;
    border: none;
    cursor: pointer;
    font: inherit;
    padding: 0;
    text-align: inherit;
}

.emoteSender {
    appearance: none;
    background: none;
    border: none;
    padding: 0;
    color: inherit;
    cursor: pointer;
    font-family: inherit;
    font-size: inherit;
    font-style: inherit;
    font-weight: inherit;
    letter-spacing: inherit;
    line-height: inherit;
    text-align: inherit;
}

.editedMarker:focus-visible,
.bodyAction:focus-visible,
.emoteSender:focus-visible {
    outline: 2px solid var(--cpd-color-border-focused);
    outline-offset: 2px;
    border-radius: var(--cpd-space-0-5x);
}
