/*
 * Copyright 2025 Element Creations Ltd.
 *
 * SPDX-License-Identifier: AGPL-3.0-only OR GPL-3.0-only OR LicenseRef-Element-Commercial
 * Please see LICENSE files in the repository root for full details.
 */

.item {
    all: unset;
    width: 100%;
    padding: 12px 16px;
    border-bottom: 1px solid #e0e0e0;

    button {
        all: unset;
    }
}

.itemSelected {
    background-color: #559f24;
}

.group {
    width: 100%;
    padding: 8px;
    background-color: #00adad;
    border: 1px solid lightgrey;
    font-weight: "bold";

    button {
        all: unset;
    }
}

.groupSelected {
    background-color: #559f24;
}
