/*
Copyright 2025 New Vector Ltd.

SPDX-License-Identifier: AGPL-3.0-only OR GPL-3.0-only OR LicenseRef-Element-Commercial
Please see LICENSE files in the repository root for full details.
*/

import React from "react";
import { render } from "jest-matrix-react";

import InviteProgressBody from "../../../../../src/components/views/dialogs/InviteProgressBody.tsx";

describe("InviteProgressBody", () => {
    it("should match snapshot", () => {
        const { asFragment } = render(<InviteProgressBody />);
        expect(asFragment()).toMatchSnapshot();
    });
});
