/*
 * Copyright 2026 Element Creations Ltd.
 *
 * SPDX-License-Identifier: AGPL-3.0-only OR GPL-3.0-only OR LicenseRef-Element-Commercial
 * Please see LICENSE files in the repository root for full details.
 */

.readMarker {
    height: 0;
    margin: 0;
    padding: 0;
    border: 0;
}

.line {
    margin-top: 0;
    position: relative;
    top: -1px;
    z-index: 1;
    width: 99%;
    opacity: 1;
    border-top: solid 1px var(--cpd-color-icon-accent-tertiary);
    border-bottom: solid 1px var(--cpd-color-icon-accent-tertiary);
    will-change: width, opacity;
    transition:
        width 400ms easeinsine 1s,
        opacity 400ms easeinsine 1s;
}
