/*
 * Copyright 2026 Element Creations Ltd.
 *
 * SPDX-License-Identifier: AGPL-3.0-only OR GPL-3.0-only OR LicenseRef-Element-Commercial
 * Please see LICENSE files in the repository root for full details.
 */

.header {
    min-height: 60px;
    padding: 0 var(--cpd-space-4x) 0 var(--cpd-space-3x);
}

.container {
    flex: 1;
    padding-bottom: var(--cpd-space-3x);
    min-width: 0;
}

.title {
    min-width: 0;

    h1 {
        /* Remove default h1 margin */
        margin: unset;
        overflow: hidden;
        white-space: nowrap;
        text-overflow: ellipsis;
        user-select: none; /* currently supported by Chrome, Edge, Opera and Firefox */
    }
}
