/*
 * Copyright 2026 Element Creations Ltd.
 *
 * SPDX-License-Identifier: AGPL-3.0-only OR GPL-3.0-only OR LicenseRef-Element-Commercial
 * Please see LICENSE files in the repository root for full details.
 */

import { composeStories } from "@storybook/react-vite";
import { render } from "@test-utils";
import React from "react";
import { describe, it, expect } from "vitest";

import * as stories from "./ReactionsRowButtonTooltip.stories";

const { Default, ManySenders } = composeStories(stories);

describe("ReactionsRowButtonTooltip", () => {
    it("renders the tooltip with formatted senders and caption", () => {
        const { container } = render(<Default />);
        expect(container).toMatchSnapshot();
    });

    it("renders the tooltip with many senders", () => {
        const { container } = render(<ManySenders />);
        expect(container).toMatchSnapshot();
    });
});
